/*
 * Created on Tue Jan 24 10:37:39 CET 2012
 */
package com.svv.trafficcbr;
import java.util.ArrayList;
import java.util.Hashtable;

import com.svv.cbr.*;
import com.svv.cbr.cases.TrafficSituationCase;

import org.npra.ven.VenSend;
import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import org.osgi.service.event.EventConstants;
import org.osgi.service.event.EventHandler;

import com.svv.util.*;

/**
 * Initial class of the bundle in OSGi, initiates the CBR cycle
 * @author Fredrick, yvind
 *
 */
public class Activator implements BundleActivator {
	public static BundleContext bc = null;
	
	private Cbr cbr;
	private CaseBuilder caseBuilder;
	private TrafficEventHandler trafficEventHandler;
	private TrafficLightSimulator trafficLightSimulator;
	private TrafficSituationCase query;
	private QueryBuilder queryBuilder;
	private boolean paused = false;
	private boolean isRunning = true, threadHasStopped;
	
	/* (non-Javadoc)
	 * @see org.osgi.framework.BundleActivator#start(org.osgi.framework.BundleContext)
	 */
	public void start(BundleContext context) throws Exception
	{
	 	Activator.bc = context;
	 	init();
	 	startThread();
	}
	
	
	private void init()
	{
		cbr = new Cbr();
		caseBuilder = new CaseBuilder();
		trafficEventHandler = new TrafficEventHandler(caseBuilder, this);
		Hashtable<String,String> properties = new Hashtable<String,String>();
		properties.put( EventConstants.EVENT_TOPIC, "org/npra/ven/*");
		bc.registerService(EventHandler.class.getName(), trafficEventHandler, properties);
		
		trafficLightSimulator = new TrafficLightSimulator(trafficEventHandler, this); 
		queryBuilder = new QueryBuilder(trafficEventHandler);
		queryBuilder.setMode(QueryBuilder.NORMAL);
	}
	
	/**
	 * Change between the NORMAL mode that receive information from other bundles or 
	 * the TEST_QUERIES mode that read cases from file (set the path TEST_CASE_PATH in Config.java)
	 * @param mode is one of the static values in the QueryBuilder (TEST_QUERIES or NORMAL)
	 */
	public void changeQueryMode(int mode)
	{
		stopThread();
		while(!threadHasStopped)
		{
			queryBuilder.setMode(mode);
		}
		threadHasStopped = false;
		startThread();
	}
	  
	public synchronized void stopThread()
	{
		isRunning = false;
		trafficLightSimulator.stop();
	}
	  
	public synchronized void startThread()
	{
		isRunning = true;
		Active active = new Active();
		new Thread(active).start();
	}
	/**
	 * Initiates the process of storing a new traffic situation case, based on the available traffic data.
	 * @param newTrafficSituationCase
	 */
	public void retain(TrafficSituationCase newTrafficSituationCase)
	{
		cbr.retain(newTrafficSituationCase);
	}
  
	/**
	 * Changes the color of the traffic light in the SCANeR simulator relative to the value
	 * @param value A value representing the color that the traffic light should be changed to (the traffic light controlling pedestrians)
	 */
	public void changeColorSimulator(String value)
	{
		ServiceReference reference = Activator.bc.getServiceReference(VenSend.class.getName());
		VenSend service = (VenSend)Activator.bc.getService(reference);
		ArrayList<Double> solution=new ArrayList<Double>();
		solution.add((Double)3.0);
		service.sendMessage("CVIS","CHANGE_"+value,solution);
	}
	
	/**
	 * Pauses the thread to make it possible for the user to add a new case
	 */
	public void pauseThread()
	{
		trafficLightSimulator.stop();
		trafficLightSimulator.retainMode();
		paused = true;
		queryBuilder.pause();
	}
	
	public void unpauseThread()
	{
		queryBuilder.start();
		paused = false;
	}
	
	/**
	 * Reset all values in the QueryBuilder
	 */
	public void clearQueryBuilder()
	{
		queryBuilder.clear();
	}

	/**
	 * The main loop that create solutions.
	 * @author Fredrick
	 *
	 */
	public class Active implements Runnable {
		@Override
		public synchronized void run()  {
			threadHasStopped = false;
			while(isRunning){
				try {
					changeColorSimulator("RED");
					if(!paused)
					{
						query = queryBuilder.getTrafficQuery();
						if(query != null) {
							trafficLightSimulator.displayCases(query, cbr.getMatchingCases(query));
							trafficLightSimulator.showTimeWaited(queryBuilder.getTimeWaited());
							double solution = caseBuilder.getSolutionFromStringToDouble(cbr.getSolutionCase(query));
							if(solution != -1)
							{
								trafficLightSimulator.changeLight(solution);
								ReadTrafficData.storeQueryAndSolution(query, cbr.getMatchingCases(query), solution);
							}
						}
						else
						{
							trafficLightSimulator.clearTimeWaitedLine();
							trafficLightSimulator.displayNoIntentionToUserInTable();
						}
					}
					Thread.sleep(1000);
				} catch(Exception e) {
					System.out.println("Error: " + e.getMessage());
				}
			}
		threadHasStopped = true;
		}
	}
  
  /* (non-Javadoc)
   * @see org.osgi.framework.BundleActivator#stop(org.osgi.framework.BundleContext)
   */
  	public void stop(BundleContext context) throws Exception {
	}

}