package com.svv.trafficcbr;

import java.awt.BasicStroke;
import java.awt.BorderLayout;
import java.awt.Canvas;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.DecimalFormat;
import java.util.ArrayList;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;

import com.svv.cbr.cases.TrafficSituationCase;
import com.svv.util.Config;
import com.svv.util.QueryBuilder;
import com.svv.util.TrafficEventHandler;

/**
 * The simulated traffic light and the rest of the GUI.
 * @author Fredrick, yvind
 *
 */
public class TrafficLightSimulator extends Canvas{
	private Activator activator;
	private JFrame frame;
	private boolean isRed = true, isBlack = false, eventStop = true, similarityIsOne = false;
	private String timeLeft = "";
	private int timeLineX = 0;
	private double lineTime;
	
	private Image red, green, black, backBuffer;
	
	private Graphics buffer;
	private final int CANVAS_WIDTH = 200, CANVAS_HEIGHT = 400, FRAME_WIDTH = 850, FRAME_HEIGHT = 420;
	
	private JButton addCase, cancel, buttonChangeQueryMod;
	private JTextField newSolutionTextField;
	private TrafficSituationCase query = null;
	private ButtonListener buttonListener;
	
	private JTable caseTable;
	private TableValues tableValues;
	
	/**
	 * 
	 * @param trafficEventHandler Used to bind the keyEvents to the GUI
	 * @param activator The Activator
	 */
	public TrafficLightSimulator(TrafficEventHandler trafficEventHandler, Activator activator)
	{
		this.activator = activator;
		addKeyListener(trafficEventHandler);
		this.setSize(CANVAS_WIDTH, CANVAS_HEIGHT);
		frame = new JFrame();
		frame.addKeyListener(trafficEventHandler);
		frame.setSize(FRAME_WIDTH, FRAME_HEIGHT);
        frame.setVisible(true);
        frame.setResizable(false);
        
        frame.setLayout(new FlowLayout(FlowLayout.LEFT));
        
        Container c = frame.getContentPane();
        c.add(this);
        c.add(createDisplayCaseView());
        
        initPic();
        backBuffer = createImage( this.getWidth(), this.getHeight() );
        buffer = backBuffer.getGraphics();
        buffer.setColor( Color.white );
        repaint();
	}

	/**
	 * The rest of the GUI, except of the traffic light
	 * @return A JPanel with the GUI
	 */
	private JPanel createDisplayCaseView()
	{
		buttonListener = new ButtonListener();
		
		JPanel p = new JPanel();
		p.setLayout(new BorderLayout());
		
		tableValues = new TableValues();
		caseTable = new JTable(tableValues);
		caseTable.getTableHeader().setFont(new Font("Serif", Font.PLAIN,  10));
		caseTable.setDefaultRenderer(Object.class, new FirstRowFontAndColorChange());
		JScrollPane scrollPane = new JScrollPane(caseTable);
		scrollPane.setPreferredSize(new Dimension(600, 350));
		
		addCase = new JButton("New case");
		addCase.setEnabled(false);
		addCase.addActionListener(buttonListener);
		cancel = new JButton("Cancel");
		cancel.setEnabled(false);
		cancel.addActionListener(buttonListener);
		buttonChangeQueryMod = new JButton("Test mode");
		buttonChangeQueryMod.addActionListener(buttonListener);
		newSolutionTextField = new JTextField(5);
		newSolutionTextField.setEnabled(false);
		
		p.add(scrollPane, BorderLayout.CENTER);
		
		JPanel newCasePanel = new JPanel();
		newCasePanel.setLayout(new FlowLayout());
		newCasePanel.add(newSolutionTextField);
		newCasePanel.add(addCase);
		newCasePanel.add(cancel);
		newCasePanel.add(buttonChangeQueryMod);
		p.add(newCasePanel, BorderLayout.SOUTH);
		return p;
	}
	
	/**
	 * Change the font and background color of the first row in the JTable
	 * @author Fredrick, yvind
	 *
	 */
	private class FirstRowFontAndColorChange extends DefaultTableCellRenderer 
	{  
		@Override
		public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) 
		{  
			JLabel parent = (JLabel) super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);  
			
			if(row == 0) 
			{
				parent.setBackground(new Color(218, 220, 243));
				parent.setFont(parent.getFont().deriveFont(Font.BOLD));  
			}
			else
			{
				parent.setBackground(null);
			}
			return parent;  
		}
	}
	
	/**
	 * Used to update the rows and columns in the JTable.
	 * @author Fredrick, yvind
	 *
	 */
	private class TableValues extends AbstractTableModel
	{
		String[] columnNames = {"TrafficFlow","VehicleDetected", "SpeedOfDetectedVehicle", 
				"PositiveIntentions", "TimeWaited", "PedestrianType", "Solution", "Similarity"};
		private String[][] table = {{"","","","No intention","","","",""}};
		
		/**
		 * Update the table that contain all the cases.
		 * @param table new table data to show
		 */
		public void overrideTable(String[][] table)
		{
			this.table = table;
		}
		
		public void clearTable()
		{
			table = new String[][]{{"","","","No intention","","","",""}};
		}
		
		@Override
		public String getColumnName(int column) {
			return columnNames[column];
		}
		
		@Override
		public int getRowCount() {
			return table.length;
		}

		@Override
		public int getColumnCount() {
			return table[0].length;
		}

		@Override
		public Object getValueAt(int rowIndex, int columnIndex) {
			return table[rowIndex][columnIndex];
		}
	}
	
	/**
	 * Activate GUI required to store a new case
	 */
	public void retainMode()
	{
		if(!similarityIsOne)
		{
			addCase.setEnabled(true);
			newSolutionTextField.setEnabled(true);
		}
		cancel.setEnabled(true);
	}
	
	/**
	 * Deactivate GUI required to store a new case 
	 */
	public void normalMode()
	{
		addCase.setEnabled(false);
		newSolutionTextField.setText("");
		newSolutionTextField.setEnabled(false);
		cancel.setEnabled(false);
	}
	
	/**
	 * Create a new row in the caseTableArray
	 * @param caseTable a table with this format String[row][0-7]
	 * @param row The row in the JTable
	 * @param trafficCase A TrafficSitatuonCase that is filled into the row in the caseTable
	 */
	private void addRowInTable(String[][] caseTable, int row, TrafficSituationCase trafficCase)
	{
		caseTable[row][0] = trafficCase.getTrafficFlow();
		caseTable[row][1] = trafficCase.getVehicleDetected()+"";
		caseTable[row][2] = trafficCase.getSpeedOfDetectedVehicle();
		caseTable[row][3] = trafficCase.getPositiveIntentions();
		caseTable[row][4] = trafficCase.getTimeWaited();
		caseTable[row][5] = trafficCase.getPedestrianType();
		caseTable[row][6] = trafficCase.getSolution();
		double similarity = trafficCase.getSimilarity();
		if(similarity == 0)
		{
			caseTable[row][7] = "";
		}
		else
		{
			DecimalFormat df = new DecimalFormat("#.###");
			caseTable[row][7] = df.format(similarity)+"";
		}
	}
	
	/**
	 * Display the query case and the retrieved cases in the table
	 * @param query Case created from the sensor data
	 * @param trafficCases The retrieved cases
	 */
	public void displayCases(TrafficSituationCase query, ArrayList<TrafficSituationCase> trafficCases)
	{
		String[][] newCaseTable = new String[trafficCases.size()+1][8];
		addRowInTable(newCaseTable, 0,query);
		
		int i = 1;
		for(TrafficSituationCase trafficCase : trafficCases)
		{
			addRowInTable(newCaseTable, i, trafficCase);
			i++;
		}
		
		tableValues.overrideTable(newCaseTable);
		tableValues.fireTableDataChanged();
	}
	/**
	 * Initialize the pictures used by the traffic light (Reduce flickering). 
	 */
	public void initPic()
	{
        red  = Toolkit.getDefaultToolkit().getImage(Config.IMG_TRAFFIC_LIGHT_RED);
        green = Toolkit.getDefaultToolkit().getImage(Config.IMG_TRAFFIC_LIGHT_GREEN);
        black = Toolkit.getDefaultToolkit().getImage(Config.IMG_TRAFFIC_LIGHT_BLACK);
	}
	
	/**
	 * Update the traffic light graphics
	 */
	public void update(Graphics g)
	{
		buffer.clearRect(0, 0, this.getWidth(), this.getHeight() );
		drawTimeLine();
		colorChange();
		g.drawImage( backBuffer, 0, 0, this );
	}
	
    public void paint(Graphics g)
    {
    	update(g);
    }
    
	private double convertNumber(double startNewRange,double endNewRange,double startOldRange, double endOldRange,double numberToConvert){
		return startNewRange + (numberToConvert-startOldRange)*(endNewRange-startNewRange)/(endOldRange-startOldRange);
	}
	
	/**
	 * Draw the line that indicates how long a pedestrian has waited.
	 */
	private void drawTimeLine()
	{
    	if(timeLineX != 0)
    	{
        	
            Graphics2D g2D = (Graphics2D) buffer;
            g2D.setStroke(new BasicStroke(10F));
            g2D.setColor(new Color(7,76,2));
            g2D.drawLine(10, 360, timeLineX, 360);
        	
            g2D.setColor(Color.BLACK);
            g2D.setFont(new Font("SansSerif", Font.BOLD, 10));
            g2D.drawString((int)lineTime+"s", timeLineX+6, 363);
    	}
	}
	
	/**
	 * Clear the waiting time line.
	 */
	public void clearTimeWaitedLine()
	{
		timeLineX = 0;
		repaint();
	}
    
	/**
	 * Convert how long a pedestrian has waited to X coordinates, which is used to draw the timeLine.
	 * @param time The pedestrian that have waited the longest time
	 */
    public void showTimeWaited(double time)
    {
    	timeLineX = (int)convertNumber(10, this.getWidth()-10, 0, 60, time);
    	this.lineTime = time;
    	repaint();
    }
    
    /**
     * Display the right image, based on the color of the traffic light.
     */
    private void colorChange()
    {
    	if(isRed)
    	{
        	buffer.drawImage(red,0,0,this);
    	}
    	else if(!isRed)
    	{
    		if(isBlack)
    		{
    			buffer.drawImage(black,0,0,this);
    		}
    		else
    		{
    			buffer.drawImage(green,0,0,this);
    		}
        	
    		buffer.setColor(Color.BLACK);
        	buffer.setFont(new Font("SansSerif", Font.BOLD, 26));
        	buffer.drawString(timeLeft, 0, 360);
    	}
    }
    
    public void displayNoIntentionToUserInTable()
    {
		tableValues.clearTable();
		tableValues.fireTableDataChanged();
    }
	
    /**
     * Change the traffic light color. 
     * First 3 seconds yellow, green time, blinking green and than 3 seconds yellow.
     * @param time Crossing time
     */
    public void changeLight(double time)
	{
    	clearTimeWaitedLine();
    	//YELLOW
		long start = System.currentTimeMillis();
		while(eventStop)
		{
			activator.changeColorSimulator("AMBER");
			try
			{
				Thread.sleep(1000);
			}catch(Exception e){}
			double now = (System.currentTimeMillis() - start)/1000.0;
			if(now > 3.0)
			{
				break;
			}
			System.out.println("Vehicle: RED/YELLOW Pedestrian: RED");
		}
		
		System.out.println("Pedestrian: GREEN " + time);
		eventStop = true;
		isRed = false;
		isBlack = false;
		double maxTimeLeft = time+2;
		timeLeft = "" + (int)maxTimeLeft;
		repaint();
		
		start = System.currentTimeMillis();
		
		//Green light for pedestrians
		while(eventStop)
		{
			activator.changeColorSimulator("GREEN");
			try
			{
				Thread.sleep(500);
			}
			catch(Exception e){}
			double now = (System.currentTimeMillis() - start)/1000.0;
			System.out.println("GREEN "+ (int)now);
			timeLeft = "" + (int)(maxTimeLeft - (int)now);
			repaint();
			if((time/2)+2 < now)
			{
				break;
			}
		}
		start = System.currentTimeMillis();
		maxTimeLeft = time/2;
		//Flashing green light for pedestrians
		while(eventStop)
		{
			activator.changeColorSimulator("GREEN");
			try
			{
				Thread.sleep(500);
			}
			catch(Exception e){}
			double now = (System.currentTimeMillis() - start)/1000.0;
			System.out.println("GREEN FLASHING "+ (int)now);
			timeLeft = "" + (int)(maxTimeLeft - (int)(now));
			
			if(isBlack) 
			{
				isBlack = false;
			} 
			else isBlack = true;
			
			repaint();
			if((time/2) < now)
			{
				break;
			}
		}

		isRed = true;
		repaint();
		System.out.println("Pedestrian: RED");
		
		//Vehicles and pedestrians get red to clear the crossing.
		start = System.currentTimeMillis();
		activator.clearQueryBuilder();
		while(eventStop)
		{
			activator.changeColorSimulator("AMBER");
			try
			{
				Thread.sleep(1000);
			}catch(Exception e){}
			double now = (System.currentTimeMillis() - start)/1000.0;
			if(now > 3.0)
			{
				break;
			}
			System.out.println("Vehicle: RED/YELLOW Pedestrian: RED");
		}
		displayNoIntentionToUserInTable();
	}
    
    /**
     * Stop the changeLight method, because all the loops become false
     */
	public void stop()
	{
		eventStop = false;
	}
	
	/**
	 * All events triggered by the buttons.
	 * @author Fredrick, yvind
	 *
	 */
	private class ButtonListener implements ActionListener
	{

		@Override
		public void actionPerformed(ActionEvent e) {
			if(e.getSource() == cancel)
			{
				normalMode();
				activator.unpauseThread();
			}
			else if(e.getSource() == addCase)
			{
				if(newSolutionTextField.getText() != "" && query != null)
				{
					query.setSolutionProposedTransitionTime(newSolutionTextField.getText());
					activator.retain(query);
					normalMode();
					activator.startThread();
				}
			}
			else if(e.getSource() == buttonChangeQueryMod)
			{
				if(buttonChangeQueryMod.getText().equals("Test mode"))
				{
					activator.changeQueryMode(QueryBuilder.TEST_QUERIES);
					buttonChangeQueryMod.setText("Normal mode");
				}
				else if(buttonChangeQueryMod.getText().equals("Normal mode"))
				{
					activator.changeQueryMode(QueryBuilder.NORMAL);
					buttonChangeQueryMod.setText("Test mode");
				}
			}
		}
	}
}
