package com.svv.util;

import java.util.HashMap;
import java.util.Map;

import com.svv.cbr.cases.TrafficSituationCase;

/**
 * Use lists that is initialized in this method to convert data
 * @author Fredrick, yvind
 *
 */
public class CaseBuilder {
	
	HashMap<String, int[]> trafficFlow, speedOfDetectedVehicle, positiveIntentions, 
	waitingTime;
	
	HashMap<String, Double> proposedTransitionTime;
	
	public CaseBuilder()
	{
		init();
	}
	
	/**
	 * Convert a solution from continuous data to nominal data
	 * @param solution as continuous
	 * @return a transition time
	 */
	public double getSolutionFromStringToDouble(String solution)
	{
		return proposedTransitionTime.get(solution);
	}
	
	/**
	 * Convert nominal data to a case with continuous data
	 * @param valueTrafficFlow vehicles passed the last five minutes
	 * @param valueVehicleDetected true if vehicle is detected
	 * @param valueSpeedOfDetectedVehicle speed of the detected vehicle
	 * @param valuePositiveIntentions number of pedestrians waiting
	 * @param valueWaitingTime the time the first pedestrian has waited
	 * @param pedestrianType the slowest pedestrian detected. Possible values "SlowGroup", "Slow", "Normal", "Fast", "VeryFast"
	 * @return a query case with data that matches the case base
	 */
	public TrafficSituationCase createCase(double valueTrafficFlow, boolean valueVehicleDetected,double valueSpeedOfDetectedVehicle, 
			double valuePositiveIntentions, double valueWaitingTime, String pedestrianType)
	{
		try
		{
			return new TrafficSituationCase(getFeatureValue(trafficFlow, valueTrafficFlow), valueVehicleDetected,
					getFeatureValue(speedOfDetectedVehicle, valueSpeedOfDetectedVehicle), getFeatureValue(positiveIntentions, valuePositiveIntentions), 
					getFeatureValue(waitingTime, valueWaitingTime), pedestrianType);
		}catch(NullPointerException e){
			System.out.println(e.getMessage());
		}
		return null;
	}
	
	/**
	 * Convert a nominal data to continuous data
	 * @param hashMap with the feature range
	 * @param value to convert
	 * @return converted value
	 */
	private String getFeatureValue(HashMap<String, int[]> hashMap,double value)
	{
		for (Map.Entry<String, int[]> entry: hashMap.entrySet()) {
			if(entry.getValue()[0] <= value && value < entry.getValue()[1])
			{
				return entry.getKey();
			}
		}
		return null;
	}
	
	/**
	 * Initialize all feature ranges
	 */
	private void init()
	{
		trafficFlow = new HashMap<>();
		trafficFlow.put("Low", new int[]{0,34});
		trafficFlow.put("Normal", new int[]{34,88});
		trafficFlow.put("High", new int[]{88,Integer.MAX_VALUE});
		
		speedOfDetectedVehicle = new HashMap<>();
		speedOfDetectedVehicle.put("null", new int[] {-1, 0});
		speedOfDetectedVehicle.put("Slow", new int[] {0,45});
		speedOfDetectedVehicle.put("Normal", new int[] {45,55});
		speedOfDetectedVehicle.put("Fast", new int[] {55,Integer.MAX_VALUE});
		
		positiveIntentions = new HashMap<>();
		positiveIntentions.put("Few", new int[]{1,3});
		positiveIntentions.put("Normal", new int[]{3,7});
		positiveIntentions.put("High", new int[]{7,Integer.MAX_VALUE});
		
		waitingTime = new HashMap<>();
		waitingTime.put("Short", new int[]{0,16});
		waitingTime.put("Medium", new int[]{16,50});
		waitingTime.put("Long", new int[]{50,Integer.MAX_VALUE});
		
		proposedTransitionTime = new HashMap<>();
		proposedTransitionTime.put("null", -1.0);
		proposedTransitionTime.put("LessLessTime", 3.75);
		proposedTransitionTime.put("LessTime", 5.0);
		proposedTransitionTime.put("Normal", 6.25);
		proposedTransitionTime.put("ExtraTime", 7.5);
		proposedTransitionTime.put("ExtraExtraTime", 12.0);
	}
}
