package com.svv.util;
import java.io.*;
import java.util.ArrayList;
import com.svv.cbr.cases.TrafficSituationCase;

/**
 * Help methods to read and write to files
 * @author Fredrick, yvind
 *
 */
public class ReadTrafficData {
	
	private ReadTrafficData(){}
	
	public static final ArrayList<String[]> getTestCaseBase()
	{
		ArrayList<String[]> trafficCases = new ArrayList<>();
		
		try
		{
			FileReader fstream = new FileReader(Config.TEST_CASE_PATH);
			BufferedReader br = new BufferedReader(fstream);	
			String strLine;

			while ((strLine = br.readLine()) != null) 
			{
				String [] splittedLine = strLine.split("\t");
				trafficCases.add(splittedLine);
			}
			br.close();
		} catch (Exception e)
		{
			System.out.println("Cannot read detected vehicles from file!");
		}
		return trafficCases;
	}
	public static final  ArrayList<String[]> readPedestrianData(String path, long startTime)
	{
		//A hash map containing a key which is the time the pedestrians enters, and a value that is how many pedestrians enter
		ArrayList<String[]> pedestriansAtTimestamp = new ArrayList<>();
		try
		{
			FileReader fstream = new FileReader(path);
			BufferedReader br = new BufferedReader(fstream);	
			String strLine;
			String typeOfPedestrian = "";
			while ((strLine = br.readLine()) != null) 
			{
				String [] splittedLine = strLine.split("\t");
				typeOfPedestrian = splittedLine[1];
				pedestriansAtTimestamp.add(new String[] {getTimeToSendPedestrian(splittedLine[0], startTime)+"", typeOfPedestrian});
			}
			br.close();
		} catch (Exception e)
		{
			System.out.println("Could not read pedestrian data from file!");
		}
		return pedestriansAtTimestamp;
		
	}
	
	private static long getTimeToSendPedestrian(String str, long startTime) 
	{
		String[] splittedString = str.split(":");
		
		int hours = Integer.parseInt(splittedString[0]);
		int minutes = Integer.parseInt(splittedString[1]);
		int seconds = Integer.parseInt(splittedString[2]);
		
		long numberOfMilliseconds = ((hours * 3600) + (minutes * 60) + seconds) * 1000;

		return startTime + numberOfMilliseconds;
	}
	
	/**
	 * Store query and the solution when the traffic light change.
	 * Used to evaluate of the system.
	 * @param query the case used to retrieve solutions
	 * @param solutionCases all retrieved solutions
	 * @param solution the solution that is used
	 */
	public static void storeQueryAndSolution(TrafficSituationCase query, ArrayList<TrafficSituationCase> solutionCases, double solution)
	{
		String text = query.toString() +"\r\n";
		
		for(TrafficSituationCase solutionCase : solutionCases)
		{
			text += solutionCase.toString() + " " + solutionCase.getSolution() + 
					" " + solutionCase.getSimilarity()+"\r\n";
		}
		text += "Solution: " + solution +"\r\n\r\n";
		
		writeToFile("C:\\Users\\Fredrick\\Desktop\\all_solutions_from_senario.txt", text);
	}
	
	/**
	 * Write a String to a file
	 * @param path location of the file
	 * @param text Will store this text
	 */
	public static void writeToFile(String path, String text) {
		try {
			FileWriter outFile = new FileWriter(path, true);
			outFile.write(text);
			outFile.close();
		} catch (Exception e) {
			System.out.println("TrafficEventhandler error: Could not write to file "+path);
		}
		
	}
}
