/*
 * myCBR License 2.0
 *
 * Copyright (c) 2010
 * Thomas Roth-Berghofer, Armin Stahl & Deutsches Forschungszentrum f&uuml;r
 * K&uuml;nstliche Intelligenz DFKI GmbH.
 * All rights reserved.
 *
 * myCBR is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Since myCBR uses some modules, you should be aware of their licenses for
 * which you should have received a copy along with this program, too.
 *
 * endOfLic */

package de.dfki.mycbr.core.retrieval;

import java.util.HashMap;

import de.dfki.mycbr.core.DefaultCaseBase;
import de.dfki.mycbr.core.Project;
import de.dfki.mycbr.core.casebase.Case;
import de.dfki.mycbr.core.casebase.Instance;
import de.dfki.mycbr.core.model.Concept;
import de.dfki.mycbr.core.similarity.Similarity;

/**
 * @author myCBR Team
 *
 */
public class Retrieval extends HashMap<Case, Similarity> {

    /**
     *
     */
    private static final long serialVersionUID = 2656679620557431799L;

    /**
     *
     */
    private Instance query;

    /**
     *
     */
    private RetrievalEngine re;

    /**
     *
     */
    private Project p;

    /**
     *
     * @param c the query should be an instance of this concept
     */
    public Retrieval(final Concept c) {
        p = c.getProject();
        try {
			query = new Instance(c, "query");
		} catch (Exception e) {
			e.printStackTrace();
		}
        re = new SequentialRetrieval(p, this);
    }

    /**
     *
     * @throws Exception if something goes wrong during retrieval
     */
    public final void start() throws Exception {
        DefaultCaseBase cb = (DefaultCaseBase) p.getCaseBases().values()
            .iterator().next();
        re.retrieve(cb, query);
    }

    /**
     * @since 3.0.0 BETA 0.2
     */
    public void setRetrievalEngine(RetrievalEngine re) {
    	this.re = re;
    }
    
    /**
     * @since 3.0.0 BETA 0.2
     */
    public Instance getQuery() {
    	return query;
    }
}
