/*
 * myCBR License 2.0
 *
 * Copyright (c) 2009
 * Thomas Roth-Berghofer, Armin Stahl & Deutsches Forschungszentrum f&uuml;r K&uuml;nstliche Intelligenz DFKI GmbH
 * Further contributors: myCBR Team (see http://mycbr-project.net/contact.html for further information 
 * about the myCBR Team). 
 * All rights reserved.
 *
 * myCBR is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Since myCBR uses some modules, you should be aware of their licenses for
 * which you should have received a copy along with this program, too.
 * 
 * endOfLic */

package de.dfki.mycbr.io;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.text.SimpleDateFormat;
import java.util.Map;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import de.dfki.mycbr.core.ICaseBase;
import de.dfki.mycbr.core.Project;
import de.dfki.mycbr.core.casebase.Attribute;
import de.dfki.mycbr.core.casebase.Case;
import de.dfki.mycbr.core.casebase.Instance;
import de.dfki.mycbr.core.casebase.SimpleAttribute;
import de.dfki.mycbr.core.casebase.SymbolAttribute;
import de.dfki.mycbr.core.explanation.ConceptExplanation;
import de.dfki.mycbr.core.model.AttributeDesc;
import de.dfki.mycbr.core.model.BooleanDesc;
import de.dfki.mycbr.core.model.Concept;
import de.dfki.mycbr.core.model.ConceptDesc;
import de.dfki.mycbr.core.model.DateDesc;
import de.dfki.mycbr.core.model.DescriptionEnum;
import de.dfki.mycbr.core.model.FloatDesc;
import de.dfki.mycbr.core.model.IntegerDesc;
import de.dfki.mycbr.core.model.IntervalDesc;
import de.dfki.mycbr.core.model.SimpleAttDesc;
import de.dfki.mycbr.core.model.StringDesc;
import de.dfki.mycbr.core.model.SymbolDesc;
import de.dfki.mycbr.core.similarity.AdvancedFloatFct;
import de.dfki.mycbr.core.similarity.AdvancedIntegerFct;
import de.dfki.mycbr.core.similarity.AmalgamationFct;
import de.dfki.mycbr.core.similarity.DateFct;
import de.dfki.mycbr.core.similarity.FloatFct;
import de.dfki.mycbr.core.similarity.FunctionEnum;
import de.dfki.mycbr.core.similarity.ISimFct;
import de.dfki.mycbr.core.similarity.IntegerFct;
import de.dfki.mycbr.core.similarity.IntervalFct;
import de.dfki.mycbr.core.similarity.OrderedSymbolFct;
import de.dfki.mycbr.core.similarity.Similarity;
import de.dfki.mycbr.core.similarity.StringFct;
import de.dfki.mycbr.core.similarity.SymbolFct;
import de.dfki.mycbr.core.similarity.TaxonomyFct;
import de.dfki.mycbr.core.similarity.TaxonomyNode;

/**
 * Not implemented yet.
 * 
 * @author myCBR Team
 * 
 */
public class XMLExporter {

	public static final String PROJECT_FILE_EXTENSION = ".myCBR";
	public static final String CB_FILE_EXTENSION = ".myCB";

	public static void save(Project prj, String path) {
		try {

			OutputStream fout = new FileOutputStream(new File(path.replace(
					".zip", ".myCBR")));
			OutputStream bout = new BufferedOutputStream(fout);
			OutputStreamWriter out = new OutputStreamWriter(bout, "8859_1");

			out.write("<?xml version=\"1.0\" ");
			out.write("encoding=\"ISO-8859-1\"?>\r\n");
			out.write("<Project name=\"" + encode(prj.getName())
					+ "\" author=\"" + encode(prj.getAuthor())
					+ "\" >\r\n");

			// save special value description
			out.write("<svs>\r\n");
			for (String svEntry : prj.getSpecialValueDesc().getAllowedValues()) {
				out.write("  <sv name=\"" + encode(svEntry) + "\"/>\r\n");
			}
			// save special value function
			for (ISimFct fct : prj.getSpecialValueDesc().getSimFcts()) {
				if (fct instanceof SymbolFct) {
					SymbolFct f = (SymbolFct) fct;
					out.write("<fct name=\"" + encode(f.getName())
							+ "\" type=\"" + FunctionEnum.Symbol + "\""
							+ " mt=\"" + f.getMultipleConfig().getMainType()
							+ "\" " + "r=\"" + f.getMultipleConfig().getReuse()
							+ "\" " + "t=\"" + f.getMultipleConfig().getType()
							+ "\" " + "symm=\"" + f.isSymmetric() + "\">\r\n");
					for (SymbolAttribute a : f.getDesc().getSymbolAttributes()) {
						out.write("<qsym name=\"" + a.getValue() + "\">\r\n");
						for (SymbolAttribute a2 : f.getDesc()
								.getSymbolAttributes()) {
							Similarity sim = f.calculateSimilarity(a, a2);
							if (!a2.equals(a)) {
								if (sim.getValue() != 0.0) {
									out.write("<csym name=\"" + a2.getValue()
											+ "\" sim=\"" + sim + "\"/>\r\n");
								}
							} else if (sim.getRoundedValue() != 1.0) {
								out.write("<csym name=\"" + a2.getValue()
										+ "\" sim=\"" + sim + "\"/>\r\n");
							}
						}
						out.write("</qsym>\r\n");
					}
					out.write("</fct>\r\n");
				}
			}
			out.write("</svs>\r\n");

			// save model
			out.write("<model>\r\n");
			for (Concept c : prj.getSubConcepts().values()) {
				saveConcept(out, c, "  ");
			}
			out.write("</model>\r\n");

			// save inheritance description
			out.write("<hierarchy>\r\n");
			saveTaxonomy(out, prj.getInhFct(), "  ");
			out.write("</hierarchy>\r\n");
			for (ICaseBase cb: prj.getCaseBases().values()) {
				out.write("<cases no=\"" + cb.getCases().size()
						+ "\" cb=\"" + cb.getName() + "\"/>\r\n");
			}
			out.write("</Project>\r\n");

			out.flush();
			out.close();

			// **********************************************************
			// ******************* WRITE CB FILE ************************
			// **********************************************************
			File cbFile = new File(path.replace(".zip", ".myCB"));
			fout = new FileOutputStream(cbFile);
			bout = new BufferedOutputStream(fout);
			out = new OutputStreamWriter(bout, "8859_1");

			out.write("<?xml version=\"1.0\" ");
			out.write("encoding=\"ISO-8859-1\"?>\r\n");
			out.write("<Project name=\"" + encode(prj.getName())
					+ "\" author=\"" + encode(prj.getAuthor())
					+ "\" >\r\n");

			for (ICaseBase cb: prj.getCaseBases().values()) {
				out.write("<cb name=\"" + encode(cb.getName()) + "\" />\r\n");
			}
			
			// save instances
			for (Concept c : prj.getSubConcepts().values()) {
				out.write("<instances name=\"" + encode(c.getName()) + "\">\r\n");
				saveInstances(out, c);
				out.write("</instances>\r\n");
			}

			out.write("</Project>\r\n");

			out.flush();
			out.close();

			// save explanations
			File expFile = new File(path.replace(".zip", ".myExp"));
			fout = new FileOutputStream(expFile);
			bout = new BufferedOutputStream(fout);
			out = new OutputStreamWriter(bout, "8859_1");

			out.write("<?xml version=\"1.0\" ");
			out.write("encoding=\"ISO-8859-1\"?>\r\n");
			out.write("<Project name=\"" + encode(prj.getName())
					+ "\" author=\"" + encode(prj.getAuthor())
					+ "\" >\r\n");

			for (ConceptExplanation e : prj.getExplanationManager()
					.getExplanations()) {
				if (e.getExplainable() instanceof Concept) {
					out.write("<exp obj=\""
							+ encode(e.getExplainable().getName())
							+ "\" type=\"" + e.getExplainable().getExpType()
							+ "\" >\r\n");
				} else if (e.getExplainable() instanceof AttributeDesc) {
					out.write("<exp obj=\""
							+ encode(e.getExplainable().getName())
							+ "\" c=\""
							+ encode(((AttributeDesc) e.getExplainable())
									.getOwner().getName()) + "\" type=\""
							+ e.getExplainable().getExpType() + "\" >\r\n");
				} else if (e.getExplainable() instanceof SimpleAttribute) {
					out.write("<exp obj=\""
							+ encode(e.getExplainable().getName())
							+ "\" c=\""
							+ encode(((SimpleAttribute) e.getExplainable())
									.getAttributeDesc().getOwner().getName())
							+ "\" desc=\""
							+ encode(((SimpleAttribute) e.getExplainable())
									.getAttributeDesc().getName())
							+ "\" type=\"" + e.getExplainable().getExpType()
							+ "\" >\r\n");
				} else if (e.getExplainable() instanceof Instance) {
					out.write("<exp obj=\""
							+ encode(e.getExplainable().getName())
							+ "\" c=\""
							+ encode(((Instance) e.getExplainable())
									.getConcept().getName()) + "\" type=\""
							+ e.getExplainable().getExpType() + "\" >\r\n");
				}
				out.write("  <desc>\r\n");
				out.write(encode(e.getDescription()));
				out.write("  </desc>\r\n");
				for (String url : e.getLinks()) {
					out.write("  <link url=\"" + url + "\" />\r\n");
				}
				out.write("</exp>\r\n");
			}
			out.write("</Project>\r\n");

			out.flush();
			out.close();
		} catch (IOException e) {
			e.printStackTrace();
		}

		zip(path);
	}

	/**
	 * @param path
	 */
	private static void zip(String path) {
		new File(path).delete(); // delete old zip file

		// generate new zip file
		FileOutputStream dest = null;
		try {
			dest = new FileOutputStream(path);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		ZipOutputStream out = new ZipOutputStream(
				new BufferedOutputStream(dest));
		File f1 = new File(path.replace(".zip", ".myCBR"));
		File f2 = new File(path.replace(".zip", ".myCB"));
		File f3 = new File(path.replace(".zip", ".myExp"));

		addFile(f1, out);
		addFile(f2, out);
		addFile(f3, out);

		new File(path.replace(".zip", ".myCBR")).delete();
		new File(path.replace(".zip", ".myCB")).delete();
		new File(path.replace(".zip", ".myExp")).delete();
		try {
			out.close();
		} catch (IOException e) {
			e.printStackTrace();
		}

	}

	/**
	 * @param f1
	 */
	private static void addFile(File f, ZipOutputStream out) {
		try {
			byte data[] = new byte[2048];
			FileInputStream in = new FileInputStream(f);
			int slash = f.getName().lastIndexOf(File.separator);
			String name;

			// Add ZIP entry to output stream.
			name = f.getName().substring(slash + 1);
			out.putNextEntry(new ZipEntry(name));

			// Transfer bytes from the file to the ZIP file
			int len;
			while ((len = in.read(data)) > 0) {
				out.write(data, 0, len);
			}

			// Complete the entry
			in.close();
			out.closeEntry();

		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}

	}

	/**
	 * @param out
	 * @param c
	 */
	private static void saveInstances(OutputStreamWriter out, Concept c) {

		try {

			for (Instance i : c.getInstances()) {
				out.write("  <instance id=\"" + encode(i.getName())
						+ "\" >\r\n");
				for (Map.Entry<AttributeDesc, Attribute> entry : i
						.getAttributes().entrySet()) {
					String value = entry.getValue().getValueAsString();
					if (value != Project.UNDEFINED_SPECIAL_ATTRIBUTE) { // only write attribute value, if it is not _undefined_
																	// because this is the default value and will be set automatically
																	// if no value is given
						out.write("    <att name=\""
								+ encode(entry.getKey().getName())
								+ "\" value=\"" + encode(value) + "\" />\r\n");
					}
				}
				for (Case entry : i.getCases()) {
					out.write("    <case name=\"" + encode(entry.getID())
							+ "\" cb=\"" + encode(entry.getCB().getName())
							+ "\" date=\"" + encode(entry.getDateFormat().format(entry.getDate())) + "\" author=\"" + encode(entry.getAuthor().toString()) + "\" />\r\n");
				}
				out.write("  </instance>\r\n");
			}

		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * @param out
	 * @param c
	 * @param string
	 */
	private static void saveConcept(OutputStreamWriter out, Concept c,
			String tab) {
		try {
			out.write(tab + "<concept name=\"" + encode(c.getName()) + "\">\r\n");
			for (AttributeDesc a : c.getAttributeDescs().values()) {
				saveDesc(out, a, tab + "  ");
			}
			for (Concept sub : c.getSubConcepts().values()) {
				saveConcept(out, sub, tab + "  ");
			}
			for (AmalgamationFct f : c.getAvailableAmalgamFcts()) {
				saveAmalgam(out, f, tab + "  ", c.getActiveAmalgamFct().equals(
						f));
			}
			out.write(tab + "</concept>\r\n");
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * @param out
	 * @param f
	 * @param string
	 */
	private static void saveAmalgam(OutputStreamWriter out, AmalgamationFct f,
			String tab, boolean active) {
		try {
			out.write(tab + "<amalgam name=\"" + encode(f.getName())
					+ "\" type=\"" + f.getType() + "\" active=\"" + active
					+ "\" >\r\n");
			for (AttributeDesc d : f.getConcept().getAllAttributeDescs()
					.values()) {
				String localFctName = "";
				if (d instanceof ConceptDesc) {
					localFctName = ((AmalgamationFct) f.getActiveFct(d))
							.getName();
				} else {
					localFctName = ((ISimFct) f.getActiveFct(d)).getName();
				}
				out.write(tab + "  <entry name=\"" + encode(d.getName())
						+ "\" active=\"" + f.isActive(d) + "\" " + "fct=\""
						+ localFctName + "\" weight=\"" + f.getWeight(d)
						+ "\"/>\r\n");
			}
			out.write(tab + "</amalgam>\r\n");
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * @param out
	 * @param a
	 * @param string
	 */
	private static void saveDesc(OutputStreamWriter out, AttributeDesc a,
			String tab) {
		try {
			if (a instanceof SimpleAttDesc) {
				SimpleAttDesc s = (SimpleAttDesc) a;
				// save desc
				if (a instanceof IntegerDesc) {
					IntegerDesc d = (IntegerDesc) a;
					out.write(tab + "<desc name=\"" + encode(a.getName())
							+ "\" type=\"" + DescriptionEnum.Integer
							+ "\" min=\"" + d.getMin() + "\" max=\""
							+ d.getMax() + "\" mult=\"" + d.isMultiple()
							+ "\" >\r\n");
				} else if (a instanceof FloatDesc) {
					FloatDesc d = (FloatDesc) a;
					out.write(tab + "<desc name=\"" + encode(a.getName())
							+ "\" type=\"" + DescriptionEnum.Float
							+ "\" min=\"" + d.getMin() + "\" max=\""
							+ d.getMax() + "\" mult=\"" + d.isMultiple()
							+ "\" >\r\n");
				} else if (a instanceof BooleanDesc) {
					out.write(tab + "<desc name=\"" + encode(a.getName())
							+ "\" type=\"" + DescriptionEnum.Boolean
							+ "\" mult=\"" + a.isMultiple() + "\" >\r\n");
				} else if (a instanceof SymbolDesc) {
					SymbolDesc d = (SymbolDesc) a;
					out.write(tab + "<desc name=\"" + encode(a.getName())
							+ "\" type=\"" + DescriptionEnum.Symbol
							+ "\" mult=\"" + d.isMultiple() + "\" >\r\n");
					for (String symb : d.getAllowedValues()) {
						out.write(tab + "  <symbol value=\"" + encode(symb)
								+ "\" />\r\n");
					}
				} else if (a instanceof StringDesc) {
					out.write(tab + "<desc name=\"" + encode(a.getName())
							+ "\" type=\"" + DescriptionEnum.String
							+ "\" mult=\"" + a.isMultiple() + "\" >\r\n");
				} else if (a instanceof IntervalDesc) {
					IntervalDesc d = (IntervalDesc) a;
					out.write(tab + "<desc name=\"" + encode(a.getName())
							+ "\" type=\"" + DescriptionEnum.Interval
							+ "\" min=\"" + d.getMin() + "\" max=\""
							+ d.getMax() + "\" mult=\"" + d.isMultiple()
							+ "\" >\r\n");
				} else if (a instanceof DateDesc) {
					DateDesc d = (DateDesc) a;
					SimpleDateFormat df = d.getFormat();
					out.write(tab + "<desc name=\"" + encode(a.getName())
							+ "\" type=\"" + DescriptionEnum.Date + "\" format=\"" + df.toPattern() + "\" min=\""
							+ df.format(d.getMinDate()) + "\" max=\"" + df.format(d.getMaxDate())
							+ "\" mult=\"" + d.isMultiple() + "\" >\r\n");
				}
				// save fcts
				for (ISimFct f : s.getSimFcts()) {
					saveFct(out, f, tab.concat("  "));
				}
				out.write(tab + "</desc>\r\n");
			} else { // ConceptDesc
				ConceptDesc d = (ConceptDesc) a;
				out.write(tab + "<desc name=\"" + encode(a.getName())
						+ "\" type=\"" + DescriptionEnum.Concept + "\" concept=\""
						+ encode(d.getConcept().getName()) + "\" mult=\""
						+ d.isMultiple() + "\" >\r\n");
				// save fcts
				for (AmalgamationFct f : d.getConcept()
						.getAvailableAmalgamFcts()) {
					out.write(tab + "  <mc fct=\"" + encode(f.getName())
							+ "\" mt=\"" + d.getMultipleConfig(f).getMainType()
							+ "\" " + "r=\""
							+ d.getMultipleConfig(f).getReuse() + "\" "
							+ "t=\"" + d.getMultipleConfig(f).getType()
							+ "\" />\r\n");
				}
				out.write(tab + "</desc>\r\n");
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * TODO
	 * 
	 * @param out
	 * @param f
	 * @param concat
	 */
	private static void saveFct(OutputStreamWriter out, ISimFct f, String tab) {
		try {
			if (f instanceof AdvancedFloatFct) {
				AdvancedFloatFct fct = (AdvancedFloatFct) f;
				out.write(tab + "<fct name=\"" + encode(f.getName())
						+ "\" type=\"" + FunctionEnum.AdvancedFloat
						+ "\" mode=\"" + fct.getMode() + "\" symm=\""
						+ fct.isSymmetric() + "\" mt=\""
						+ f.getMultipleConfig().getMainType() + "\" " + "r=\""
						+ f.getMultipleConfig().getReuse() + "\" " + "t=\""
						+ f.getMultipleConfig().getType() + "\">\r\n");
				for (Map.Entry<Double, Similarity> p : fct
						.getAdditionalPoints().entrySet()) {
					out.write(tab + "  <point x=\"" + p.getKey() + "\" y=\""
							+ p.getValue().getRoundedValue() + "\" />\r\n");
				}
				out.write(tab + "</fct>\r\n");
			} else if (f instanceof AdvancedIntegerFct) {
				AdvancedIntegerFct fct = (AdvancedIntegerFct) f;
				out.write(tab + "<fct name=\"" + encode(f.getName())
						+ "\" type=\"" + FunctionEnum.AdvancedInteger
						+ "\" mode=\"" + fct.getMode() + "\" symm=\""
						+ fct.isSymmetric() + "\" mt=\""
						+ f.getMultipleConfig().getMainType() + "\" " + "r=\""
						+ f.getMultipleConfig().getReuse() + "\" " + "t=\""
						+ f.getMultipleConfig().getType() + "\">\r\n");
				for (Map.Entry<Double, Similarity> p : fct
						.getAdditionalPoints().entrySet()) {
					out.write(tab + "  <point x=\"" + p.getKey() + "\" y=\""
							+ p.getValue().getRoundedValue() + "\" />\r\n");
				}
				out.write(tab + "</fct>\r\n");
			} else if (f instanceof DateFct) {
				out.write(tab + "<fct name=\"" + encode(f.getName())
						+ "\" type=\"" + FunctionEnum.Date + "\" symm=\""
						+ f.isSymmetric() + "\" mt=\""
						+ f.getMultipleConfig().getMainType() + "\" " + "r=\""
						+ f.getMultipleConfig().getReuse() + "\" " + "t=\""
						+ f.getMultipleConfig().getType() + "\" />\r\n");
			} else if (f instanceof FloatFct) {
				FloatFct fct = (FloatFct) f;
				out.write(tab + "<fct name=\"" + encode(f.getName())
						+ "\" type=\"" + FunctionEnum.Float + "\" ltype=\""
						+ fct.getFunctionTypeL() + "\" lparam=\""
						+ fct.getFunctionParameterL() + "\" rtype=\""
						+ fct.getFunctionTypeR() + "\" rparam=\""
						+ fct.getFunctionParameterR() + "\" mode=\""
						+ fct.getMode() + "\" symm=\"" + fct.isSymmetric()
						+ "\" mt=\"" + f.getMultipleConfig().getMainType()
						+ "\" " + "r=\"" + f.getMultipleConfig().getReuse()
						+ "\" " + "t=\"" + f.getMultipleConfig().getType()
						+ "\" />\r\n");
			} else if (f instanceof IntegerFct) {
				IntegerFct fct = (IntegerFct) f;
				out.write(tab + "<fct name=\"" + encode(f.getName())
						+ "\" type=\"" + FunctionEnum.Integer + "\" ltype=\""
						+ fct.getFunctionTypeL() + "\" lparam=\""
						+ fct.getFunctionParameterL() + "\" rtype=\""
						+ fct.getFunctionTypeR() + "\" rparam=\""
						+ fct.getFunctionParameterR() + "\" mode=\""
						+ fct.getMode() + "\" symm=\"" + fct.isSymmetric()
						+ "\" mt=\"" + f.getMultipleConfig().getMainType()
						+ "\" " + "r=\"" + f.getMultipleConfig().getReuse()
						+ "\" " + "t=\"" + f.getMultipleConfig().getType()
						+ "\" />\r\n");
			} else if (f instanceof IntervalFct) {
				out.write(tab + "<fct name=\"" + encode(f.getName())
						+ "\" type=\"" + FunctionEnum.Interval + "\" symm=\""
						+ f.isSymmetric() + "\" mt=\""
						+ f.getMultipleConfig().getMainType() + "\" " + "r=\""
						+ f.getMultipleConfig().getReuse() + "\" " + "t=\""
						+ f.getMultipleConfig().getType() + "\" />\r\n");
			} else if (f instanceof OrderedSymbolFct) {
				OrderedSymbolFct fct = (OrderedSymbolFct) f;
				out.write("<fct name=\"" + encode(f.getName()) + "\" type=\""
						+ FunctionEnum.OrderedSymbol + "\"" + " mt=\""
						+ f.getMultipleConfig().getMainType() + "\" " + "r=\""
						+ f.getMultipleConfig().getReuse() + "\" " + "t=\""
						+ f.getMultipleConfig().getType() + "\" " + "symm=\""
						+ f.isSymmetric() + "\"" + " cyclic=\""
						+ fct.isCyclic() + "\" >\r\n");
				for (SymbolAttribute a : fct.getDesc().getSymbolAttributes()) {
					out.write("<qsym name=\"" + a.getValue() + "\">\r\n");
					for (SymbolAttribute a2 : fct.getDesc()
							.getSymbolAttributes()) {
						if (!a2.equals(a)) {
							Similarity sim = f.calculateSimilarity(a, a2);
							if (sim.getValue() != 0.0) {
								out.write("<csym name=\"" + a2.getValue()
										+ "\" sim=\"" + sim + "\"/>\r\n");
							}
						}
					}
					out.write("</qsym>\r\n");
				}
				IntegerFct intFct = fct.getInternalFunction();
				out.write(tab + "<intfct name=\"" + encode(intFct.getName())
						+ "\" type=\"" + FunctionEnum.Integer + "\" ltype=\""
						+ intFct.getFunctionTypeL() + "\" lparam=\""
						+ intFct.getFunctionParameterL() + "\" rtype=\""
						+ intFct.getFunctionTypeR() + "\" rparam=\""
						+ intFct.getFunctionParameterR() + "\" mode=\""
						+ intFct.getMode() + "\" symm=\""
						+ intFct.isSymmetric() + "\"  />\r\n");
				for (Map.Entry<SymbolAttribute, Integer> entry : fct.getOrder()
						.entrySet()) {
					out.write(tab + "  <order name=\""
							+ encode(entry.getKey().getValueAsString())
							+ "\" index=\"" + entry.getValue() + "\" />\r\n");
				}
				out.write("</fct>\r\n");
			} else if (f instanceof TaxonomyFct) {

				TaxonomyFct fct = (TaxonomyFct) f;
				saveTaxonomy(out, fct, tab);
			} else if (f instanceof StringFct) {
				StringFct fct = (StringFct) f;
				out.write(tab + "<fct name=\"" + encode(f.getName())
						+ "\" type=\"" + FunctionEnum.String + "\" symm=\""
						+ f.isSymmetric() + "\" config=\"" + fct.getConfig()
						+ "\" n=\"" + fct.getN() + "\" mt=\""
						+ f.getMultipleConfig().getMainType() + "\" " + "r=\""
						+ f.getMultipleConfig().getReuse() + "\" " + "t=\""
						+ f.getMultipleConfig().getType() + "\" />\r\n");
			} else if (f instanceof SymbolFct) {
				SymbolFct fct = (SymbolFct) f;
				out.write("<fct name=\"" + encode(f.getName()) + "\" type=\""
						+ FunctionEnum.Symbol + "\"" + " mt=\""
						+ f.getMultipleConfig().getMainType() + "\" " + "r=\""
						+ f.getMultipleConfig().getReuse() + "\" " + "t=\""
						+ f.getMultipleConfig().getType() + "\" " + "symm=\""
						+ f.isSymmetric() + "\">\r\n");
				for (SymbolAttribute a : fct.getDesc().getSymbolAttributes()) {
					out.write("<qsym name=\"" + a.getValue() + "\">\r\n");
					for (SymbolAttribute a2 : fct.getDesc()
							.getSymbolAttributes()) {
						Similarity sim = f.calculateSimilarity(a, a2);
						if (!a2.equals(a)) {
							if (sim.getValue() != 0.0) {
								out.write("<csym name=\"" + a2.getValue()
										+ "\" sim=\"" + sim + "\"/>\r\n");
							}
						} else if (sim.getRoundedValue() != 1.0) {
							out.write("<csym name=\"" + a2.getValue()
									+ "\" sim=\"" + sim + "\"/>\r\n");
						}
					}
					out.write("</qsym>\r\n");
				}
				out.write("</fct>\r\n");
			}
		} catch (IOException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * @param out
	 * @param desc
	 */
	private static void saveTaxonomy(OutputStreamWriter out, TaxonomyFct fct,
			String tab) {
		try {
			out.write(tab
					+ "<fct name=\""
					+ encode(fct.getName())
					+ "\" type=\""
					+ FunctionEnum.Taxonomy
					+ "\" qconfig=\""
					+ fct.getQueryConfig()
					+ "\" cconfig=\""
					+ fct.getCaseConfig()
					+ "\" top=\""
					+ fct.getTaxonomy().getTopSymbol()
					+ "\" sim=\""
					+ fct.getTaxonomy().getSimilarity(
							fct.getTaxonomy().getTopSymbol()) + "\" symm=\""
					+ fct.isSymmetric() + "\" >\r\n");

			for (Map.Entry<TaxonomyNode, TaxonomyNode> entry : fct
					.getTaxonomy().getParentMap().entrySet()) {
				out.write(tab + "  <node name=\""
						+ encode(entry.getKey().toString()) + "\" sim=\""
						+ fct.getTaxonomy().getSimilarity(entry.getKey())
						+ "\" parent=\"" + entry.getValue() + "\" />\r\n");
			}
			out.write(tab + "</fct>\r\n");
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	private static String encode(String in) {
		return in;
	}
}
