package CBR;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;


import cases.TrafficSituationCase;
import util.Config;
import util.ReadTestCaseBase;

import de.dfki.mycbr.core.ICaseBase;

import de.dfki.mycbr.core.Project;
import de.dfki.mycbr.core.casebase.Case;
import de.dfki.mycbr.core.casebase.Instance;
import de.dfki.mycbr.core.model.Concept;
import de.dfki.mycbr.core.model.FloatDesc;
import de.dfki.mycbr.core.model.SymbolDesc;
import de.dfki.mycbr.core.retrieval.Retrieval;
import de.dfki.mycbr.core.similarity.AmalgamationFct;
import de.dfki.mycbr.core.similarity.FloatFct;
import de.dfki.mycbr.core.similarity.ISimFct;
import de.dfki.mycbr.core.similarity.Similarity;
import de.dfki.mycbr.core.similarity.SymbolFct;

public class TrafficSituation{
	private SymbolDesc proposedTransitionTime;
	private Concept trafficSituation;
	private Project projectTrafficSituation;
	private ICaseBase caseBase;
	
	public TrafficSituation() throws Exception
	{
		projectTrafficSituation = new Project(Config.TRAFFIC_SITUATION);
		caseBase = projectTrafficSituation.getCB("CaseBase0");
		HashMap<String, Concept> hm = projectTrafficSituation.getAllSubConcepts();
		trafficSituation = hm.get("TrafficSituation");
		
		proposedTransitionTime = (SymbolDesc)trafficSituation.getAttributeDesc("ProposedTransitionTime");
	}
	
	public String getProposedTransitionTime(TrafficSituationCase trafficCase)throws Exception
	{
		Retrieval r = new Retrieval(trafficSituation);
		Instance q = r.getQuery();
		q.addAttribute("TrafficFlow", trafficCase.getTrafficFlow());
		q.addAttribute("VehicleDetected", trafficCase.getVehicleDetected());
		q.addAttribute("SpeedOfDetectedVehicle", trafficCase.getSpeedOfDetectedVehicle());
		q.addAttribute("PositiveIntentions", trafficCase.getPositiveIntentions());
		q.addAttribute("TimeWaited", trafficCase.getTimeWaited());
		q.addAttribute("PedestrianType", trafficCase.getPedestrianType());
		r.start();
		
		double maxSimilarity = 0;
		HashMap<String, Integer> solutionList = new HashMap<String, Integer>();

		for (Map.Entry<Case, Similarity> entry: r.entrySet()) {
			if(maxSimilarity < entry.getValue().getValue())
			{
				maxSimilarity = entry.getValue().getValue();
				String solution = entry.getKey().getAttForDesc(proposedTransitionTime).getValueAsString();
				//If a query got equal similarity with several cases, choose the solution that occurs most. 
				if(!solutionList.containsKey(solution))
				{
					solutionList.put(solution, 1);
				}
				else
				{
					int value = (int)solutionList.get(solution) + 1;
					solutionList.put(solution, value);
				}
			}
		}
		return getMaxValue(solutionList);
	}
	
	public ArrayList<TrafficSituationCase> getCasesMatchingQuery(TrafficSituationCase trafficCase) throws Exception
	{
		Retrieval r = doQuery(trafficCase);
		
		ArrayList<TrafficSituationCase> solutionTrafficCases = new ArrayList<TrafficSituationCase>();
		double maxSimilarity = 0;
		for (Map.Entry<Case, Similarity> entry: r.entrySet()) {
			double currentSimilarity = entry.getValue().getValue();
			if(currentSimilarity > maxSimilarity)
			{
				maxSimilarity = currentSimilarity;
				solutionTrafficCases.clear();
				solutionTrafficCases.add(createTrafficSituationCase(entry.getKey(), currentSimilarity));
			}
			else if(currentSimilarity == maxSimilarity)
			{
				solutionTrafficCases.add(createTrafficSituationCase(entry.getKey(), currentSimilarity));
			}
		}
		return solutionTrafficCases;
	}
	
	private TrafficSituationCase createTrafficSituationCase(Case c, double similairty)
	{
		String trafficFlow = c.getAttForDesc(trafficSituation.getAttributeDesc("TrafficFlow")).getValueAsString();
		boolean vehicleDetected = Boolean.parseBoolean(c.getAttForDesc(trafficSituation.getAttributeDesc("VehicleDetected")).getValueAsString());
		String speedOfDetectedVehicle = c.getAttForDesc(trafficSituation.getAttributeDesc("SpeedOfDetectedVehicle")).getValueAsString();
		String positiveIntentions = c.getAttForDesc(trafficSituation.getAttributeDesc("PositiveIntentions")).getValueAsString();
		String timeWaited = c.getAttForDesc(trafficSituation.getAttributeDesc("TimeWaited")).getValueAsString();
		String pedestrianType = c.getAttForDesc(trafficSituation.getAttributeDesc("PedestrianType")).getValueAsString();
		String solutionProposedTransitionTime = c.getAttForDesc(proposedTransitionTime).getValueAsString();
		
		TrafficSituationCase newCase = new TrafficSituationCase(trafficFlow, vehicleDetected, speedOfDetectedVehicle, positiveIntentions, timeWaited, pedestrianType);
		newCase.setSimilarity(similairty);
		newCase.setSolutionProposedTransitionTime(solutionProposedTransitionTime);
		return newCase;
	}
	
	private Retrieval doQuery(TrafficSituationCase trafficCase) throws Exception
	{
		Retrieval r = new Retrieval(trafficSituation);
		Instance q = r.getQuery();
		q.addAttribute("TrafficFlow", trafficCase.getTrafficFlow());
		q.addAttribute("VehicleDetected", trafficCase.getVehicleDetected());
		q.addAttribute("SpeedOfDetectedVehicle", trafficCase.getSpeedOfDetectedVehicle());
		q.addAttribute("PositiveIntentions", trafficCase.getPositiveIntentions());
		q.addAttribute("TimeWaited", trafficCase.getTimeWaited());
		q.addAttribute("PedestrianType", trafficCase.getPedestrianType());
		r.start();
		return r;
	}
	
	private String getMaxValue(HashMap<String, Integer> solutionList)
	{
		int maxValue = 0;
		String solution = "";
		for(Map.Entry<String, Integer> entry: solutionList.entrySet())
		{
			if(maxValue < entry.getValue())
			{
				maxValue = entry.getValue();
				solution = entry.getKey();
			}
		}
		return solution;
	}
	public HashMap<String,String> leaveOneOutValidation() 
	{
		Collection allCases = caseBase.getCases();
		int correctlyClassifiedCases = 0;
		double accumulatedSimilarity = 0.0;
		double correctlyClassifiedCasesWithSimilairty = 0;
		for(Object o : allCases) 
		{
            Case queryCase = (Case)o;
			caseBase.removeCase(queryCase.getID());
			
			//Problem description
			String trafficFlow = queryCase.getAttForDesc(trafficSituation.getAttributeDesc("TrafficFlow")).getValueAsString();
			boolean vehicleDetected = Boolean.parseBoolean(queryCase.getAttForDesc(trafficSituation.getAttributeDesc("VehicleDetected")).getValueAsString());
			String speedOfDetectedVehicle = queryCase.getAttForDesc(trafficSituation.getAttributeDesc("SpeedOfDetectedVehicle")).getValueAsString();
			String positiveIntentions = queryCase.getAttForDesc(trafficSituation.getAttributeDesc("PositiveIntentions")).getValueAsString();
			String timeWaited = queryCase.getAttForDesc(trafficSituation.getAttributeDesc("TimeWaited")).getValueAsString();
			String pedestrianType = queryCase.getAttForDesc(trafficSituation.getAttributeDesc("PedestrianType")).getValueAsString();
			//Correct solution
			String correctSolution = queryCase.getAttForDesc(trafficSituation.getAttributeDesc("ProposedTransitionTime")).getValueAsString();
			
			TrafficSituationCase trafficCase = new TrafficSituationCase(trafficFlow, vehicleDetected, speedOfDetectedVehicle, positiveIntentions, timeWaited, pedestrianType);
			
			try
			{
				//Gets the similarity of the first item in the ArrayList, since all have the same similarity
				ArrayList<TrafficSituationCase> ts = getCasesMatchingQuery(trafficCase);
				accumulatedSimilarity += ts.get(0).getSimilarity();

				String mostSimilarCase = Cbr.findBestTrafficSituationCase(ts);
				
				//If a case in the test case base got similarity 1, notify user.
				if(ts.get(0).getSimilarity() == 1.0)
				{
					Double[] a  = getWeights();
					for(double d : a)
					{
						System.out.print(d+" ");
					}
					System.out.print("Test case with similarity!: ");
					for(Object feature : ts)
					{
						System.out.print(feature +" ");
					}
					System.out.println();
				}
				
				correctlyClassifiedCasesWithSimilairty += calcSimilairtyBetweenSolutions(mostSimilarCase, correctSolution);
				if(mostSimilarCase.equals(correctSolution)) 
				{
					correctlyClassifiedCases += 1;
				}
				caseBase.addCase(queryCase.getInstance(), queryCase.getID());
			} catch (Exception e)
			{
				System.out.println("Could not add case!");
			}
		}
		HashMap<String, String> validationResults = new HashMap<String, String>();
		validationResults.put("averageSimilarity", "" + accumulatedSimilarity/allCases.size());
		validationResults.put("correctlyClassifiedCases", ""+ correctlyClassifiedCases);
		validationResults.put("CBSize", ""+allCases.size()); 
		validationResults.put("correctlyClassifiedCasesWithSimilairty",""+correctlyClassifiedCasesWithSimilairty);
		return validationResults;
	}
	
	public HashMap<String,String> testCases()
	{
		double accumulatedSimilarity = 0.0;
		int correctlyClassifiedCases = 0;
		double correctlyClassifiedCasesWithSimilairty = 0;
		
		ArrayList<String[]> testCaseBase = ReadTestCaseBase.getTestCaseBase();
		for(String[]  testCase: testCaseBase)
		{
			try
			{
				ArrayList<TrafficSituationCase> ts = getCasesMatchingQuery(new TrafficSituationCase(testCase[0], Boolean.parseBoolean(testCase[1]), 
						testCase[2], testCase[3], testCase[4], testCase[5]));
				
				//If a case in the test case base got similarity 1, notify user.
				if(ts.get(0).getSimilarity() == 1)
				{
					System.out.print("Test case with similarity!: ");
					for(String feature : testCase)
					{
						System.out.print(feature +" ");
					}
					System.out.println();
				}
				
				accumulatedSimilarity += ts.get(0).getSimilarity();	
				String mostSimilarCase = Cbr.findBestTrafficSituationCase(ts);
				
				String correctSolution = testCase[6];
				correctlyClassifiedCasesWithSimilairty += calcSimilairtyBetweenSolutions(mostSimilarCase, correctSolution);
				if(mostSimilarCase.equals(correctSolution)) 
				{
					correctlyClassifiedCases += 1;
				}
				
			}catch(Exception e){
				System.out.println("Error TrafficSituation-testCases(): ");
			}
		}
		HashMap<String, String> validationResults = new HashMap<String, String>();
		validationResults.put("averageSimilarity", "" + accumulatedSimilarity/testCaseBase.size());
		validationResults.put("correctlyClassifiedCases", ""+ correctlyClassifiedCases);
		validationResults.put("CBSize", ""+testCaseBase.size()); 
		validationResults.put("correctlyClassifiedCasesWithSimilairty",""+correctlyClassifiedCasesWithSimilairty);
		return validationResults;
	}
	
	public double calcSimilairtyBetweenSolutions(String mostSimilarCase ,String correctSolution)
	{
		if(mostSimilarCase.equals(correctSolution))
		{
			return 1.0;
		}
		else if(mostSimilarCase.equals("LessTime") && correctSolution.equals("LessLessTime"))
		{
			return 0.1;
		}
		else if(mostSimilarCase.equals("Normal") && correctSolution.equals("LessTime"))
		{
			return 0.1;
		}
		else if(mostSimilarCase.equals("ExtraTime") && correctSolution.equals("Normal"))
		{
			return 0.1;
		}
		else if(mostSimilarCase.equals("ExtraExtraTime") && correctSolution.equals("ExtraTime"))
		{
			return 0.1;
		}
		else
		{
			return 0;
		}
	}
	
	public Double[] getWeights()
	{
		AmalgamationFct t = trafficSituation.getActiveAmalgamFct();
		double trafficFlow = Double.parseDouble(t.getWeight(trafficSituation.getAttributeDesc("TrafficFlow")).toString());
		double vehicleDetected = Double.parseDouble(t.getWeight(trafficSituation.getAttributeDesc("VehicleDetected")).toString());
		double speedOfDetectedVehicle = Double.parseDouble(t.getWeight(trafficSituation.getAttributeDesc("SpeedOfDetectedVehicle")).toString());
		double positiveIntentions = Double.parseDouble(t.getWeight(trafficSituation.getAttributeDesc("PositiveIntentions")).toString());
		double timeWaited = Double.parseDouble(t.getWeight(trafficSituation.getAttributeDesc("TimeWaited")).toString());
		double pedestrianType = Double.parseDouble(t.getWeight(trafficSituation.getAttributeDesc("PedestrianType")).toString());
		return new Double[]{trafficFlow,vehicleDetected,speedOfDetectedVehicle,positiveIntentions,timeWaited,pedestrianType};
	}
	
	public void setWeights(Double[] weights)
	{
		AmalgamationFct t = trafficSituation.getActiveAmalgamFct();
		t.setWeight("TrafficFlow", weights[0]);
		t.setWeight("VehicleDetected", weights[1]);
		t.setWeight("SpeedOfDetectedVehicle", weights[2]);
		t.setWeight("PositiveIntentions", weights[3]);
		t.setWeight("TimeWaited", weights[4]);
		t.setWeight("PedestrianType", weights[5]);
	}
}