package EA;
import java.awt.Color;
import java.util.ArrayList;
import java.util.Random;

import javax.swing.JFrame;

import org.math.plot.Plot2DPanel;

import CBR.TrafficSituation;

public class EACore {
	private final int NUMBER_OF_UNITS = 20;
	private final int RUNS = 50;
	
	public EACore() {
		
	}
	
	private ArrayList<Unit> createUnits(int n)
	{
		ArrayList<Unit> children = new ArrayList<Unit>();
		for(int i = 0; i < n; i++)
		{
			children.add(new Unit());
		}
		return children;
	}
	
	public void start()
	{
		SelectionStrategies selectionStrategies = new SelectionStrategies(NUMBER_OF_UNITS);
		GeneticOperators geneticOperators = new GeneticOperators(0.1, 3);
		
		ArrayList<Unit> children = createUnits(NUMBER_OF_UNITS);
		ArrayList<Unit> adults = null;
		
		double[] maxFitnessInRuns = new double[RUNS];
		double[] avg = new double[RUNS];
		double[] standardDeviation = new double[RUNS];
		
		for(int i = 0; i < RUNS; i++)
		{
			adults = selectionStrategies.full_replacement(adults, children);
			ArrayList<Unit> parents = selectionStrategies.fitnessProportionate(adults);
			children = geneticOperators.crossover(parents);
			children.remove(new Random().nextInt(children.size()-1));
			children.add(geneticOperators.doElitism(adults));
			
			double[] plotValues = getPlotValues(children);
			maxFitnessInRuns[i] = plotValues[0];
			avg[i] = plotValues[1];
			standardDeviation[i] = plotValues[2];
			
			System.out.println("RUN "+i);
		}
		printMax(children,maxFitnessInRuns, avg, standardDeviation);
	}
	
	private double[] getPlotValues(ArrayList<Unit> children)
	{
		double maxFitness =0;
		double avg = 0;
		double standardDeviation = 0;
		for(Unit child : children)
		{
			double fitness = child.getFitness();
			if(maxFitness < fitness)
			{
				maxFitness = fitness;
			}
			avg += child.getFitness();
		}
		avg = avg/children.size();
		for(Unit child : children)
		{
			standardDeviation += Math.pow(child.getFitness()-avg, 2);
		}
		standardDeviation = Math.sqrt(standardDeviation/children.size());
		
		return new double[]{maxFitness,avg,standardDeviation};
	}
	
	public void printMax(ArrayList<Unit> children, double[] maxFitnessInRuns, double[] avg, double[] standardDeviation)
	{
		double maxFitness = 0;
		String maxCorrectlyCases = "";
		ArrayList<Double> weights = children.get(0).getPhenotype();
		for(Unit child : children)
		{
			double fitness = child.getFitness();
			if(maxFitness < fitness)
			{
				maxFitness = fitness;
				weights = child.getPhenotype();
				maxCorrectlyCases = child.getResultWithWeightFromTrafficSituation().get("correctlyClassifiedCases");
			}
		}
		
		System.out.println(weights.get(0)+" "+weights.get(1)+" "+weights.get(2)+" "+weights.get(3)+" "+weights.get(4)+
				" "+weights.get(5));
		System.out.println("Fitness: "+maxFitness);
		System.out.println("Correctly classified cases: " + maxCorrectlyCases);
		
		
		Plot2DPanel plot = new Plot2DPanel();
		plot.addLinePlot("Max fitness", Color.RED, maxFitnessInRuns);
		plot.addLinePlot("Avg", Color.BLUE, avg);
		plot.addLinePlot("Standard deviation", Color.GREEN, standardDeviation);
		JFrame frame = new JFrame("plotPanel");
		frame.setContentPane(plot);
		frame.setVisible(true);
		frame.setSize(500, 500);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}

	public static void main(String[] args)
	{
		EACore ea = new EACore();
		ea.start();
	}
	
}
