package EA;

import java.util.ArrayList;
import java.util.Random;

import CBR.TrafficSituation;

public class GeneticOperators {
	
	private int mutations;
	private double mutationRate;
	
	public GeneticOperators(double mutationRate, int mutations)
	{
		this.mutationRate = mutationRate;
		this.mutations = mutations;
	}
	
	public Unit doElitism(ArrayList<Unit> adults)
	{
		Unit maxParent = adults.get(0);
		for(Unit parent : adults)
		{
			double parentFitness = parent.getFitness();
			if(maxParent.getFitness() < parentFitness)
			{
				maxParent = parent;
			}
		}
		return maxParent;
	}
	
	public ArrayList<Unit> crossover(ArrayList<Unit> parents)
	{
		ArrayList<Unit> children = switchTopBottom(parents);
		
		for(int i = 0; i < children.size(); i++)
		{
			Random r = new Random();
			double precentage = (double)r.nextInt(100)/100.0;
			if(precentage <mutationRate)
			{
				Unit mutatedChild = mutation(children.get(i));
				children.remove(i);
				children.add(mutatedChild);
			}
		}
		return children;
	}
	
	public Unit mutation(Unit parent)
	{
		int[] genotype = parent.getGenotype();
		Random r = new Random();
		ArrayList<Integer> randomList = new ArrayList<Integer>();
		for(int i = 0; i < genotype.length; i++)
		{
			randomList.add(i);
		}
		for(int i = 0; i< mutations; i++)
		{
			int geneToManipulate = randomList.get(r.nextInt(randomList.size()));
			if(genotype[geneToManipulate] == 0)
			{
				genotype[geneToManipulate] = 1;
			}
			else
			{
				genotype[geneToManipulate] = 0;
			}
		}
		Unit u = new Unit(genotype);
		return u;
	}
	
	public ArrayList<Unit> switchTopBottom(ArrayList<Unit> parents)
	{
		ArrayList<Unit> children = new ArrayList<Unit>();
		for(int i = 0; i < parents.size(); i = i+2)
		{
			int[] parentOneGenotype = parents.get(i).getGenotype();
			int[] parentTwoGenotype = parents.get(i+1).getGenotype();
			
			int[] newGenotypeOne = new int[parentOneGenotype.length];
			int[] newGenotypeTwo = new int[parentOneGenotype.length];
			
			for(int j = 0; j < parentOneGenotype.length; j++)
			{
				if(j < (new Random().nextInt(parentOneGenotype.length)))
				{
					newGenotypeOne[j] = parentOneGenotype[j];
					newGenotypeTwo[j] = parentTwoGenotype[j];
				}
				else
				{
					newGenotypeOne[j] = parentTwoGenotype[j];
					newGenotypeTwo[j] = parentOneGenotype[j];
				}
			}
			Unit childOne = new Unit(newGenotypeOne);
			Unit childTwo = new Unit(newGenotypeTwo);
			children.add(childOne);
			children.add(childTwo);
		}
		return children;
	}
}
