package EA;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Random;

import CBR.TrafficSituation;

public class Unit {
	
	private int[] genotype;
	private double fitness = 0;
	private final int FEATURE_SIZE = 7;
	
	public Unit()
	{
		createGenotype();
		fitness = createFitness();
	}
	
	public Unit(int[] genotype)
	{
		this.genotype = genotype;
		fitness = createFitness();
	}
	
	private void createGenotype()
	{
		genotype = new int[FEATURE_SIZE*6];
		Random r = new Random();
		for(int i = 0; i < genotype.length; i++)
		{
			genotype[i] = r.nextInt(2);
		}
	}
	
	public int[] getGenotype()
	{
		return genotype;
	}
	
	public ArrayList<Double> getPhenotype()
	{
		ArrayList<Double> weights = new ArrayList<Double>();
		String weight = "";
		for(int i = 0; i < genotype.length; i++)
		{
			weight += genotype[i];
			if(i != 0 && (i+1) % FEATURE_SIZE == 0)
			{
				weights.add((double)(Integer.parseInt(weight, 2)/10.0));
				weight = "";
			}
		}
		return weights;
	}
	
	public HashMap<String,String> getResultWithWeightFromTrafficSituation()
	{
		TrafficSituation ts = null;
		try
		{
			ts = new TrafficSituation();
		}catch(Exception e){
			System.out.println("Error unit trafficSituation: "+e.getMessage());
		}
		
		ArrayList<Double> weights = getPhenotype();
		for(double weight : weights)
		{
			if(weight == 0.0)
			{
				return null;
			}
		}
		ts.setWeights(new Double[]{weights.get(0), weights.get(1), weights.get(2), weights.get(3), 
				weights.get(4), weights.get(5)});
		HashMap<String,String> result = ts.leaveOneOutValidation();
		return result;
	}
	
	public double createFitness()
	{
		HashMap<String,String> result = getResultWithWeightFromTrafficSituation();
		if(result == null)
		{
			return 0;
		}
		return Double.parseDouble(result.get("correctlyClassifiedCases"))/Double.parseDouble(result.get("CBSize"));
	}
	
	public double getFitness()
	{
		return 1.0/(1.0+(1.0-fitness));
	}
}


