
#define VERBOSE 1      // turn on or off debugging output

#include "Client.h"

Client::Client(int iPort, const char* pStrHost, bool* pResult)
{
	struct hostent*	he = NULL;

	m_iPort = iPort;
	m_iSock = -1;
	m_pBuffer = NULL;
	m_pBuffer2 = NULL;

	if (pResult)
		*pResult = false;

	m_pBuffer = (double *) malloc(sizeof(double) * CLIENT_BUFF_SIZE);
	if (!m_pBuffer)
	{
		perror("Server::Server, failed to malloc buffer!");
		return;
	}

	m_pBuffer2 = (double *) malloc(sizeof(double) * CLIENT_BUFF_SIZE);
	if (!m_pBuffer2)
	{
		free(m_pBuffer);
		m_pBuffer = NULL;
		perror("Server::Server, failed to malloc buffer2!");
		return;
	}

#ifdef _WIN32
	WORD wVersionRequested;
    WSADATA wsaData;
    int err;

    wVersionRequested = MAKEWORD(2, 2);

    err = WSAStartup(wVersionRequested, &wsaData);
    if (err != 0) 
	{
        perror("Client::Client, WSAStartup failed with error");
        return;
    }
#endif
	
	if (VERBOSE) 
		printf("Client: opening socket to %s on port = %d\n", pStrHost, m_iPort);

	if ((he = gethostbyname(pStrHost)) == NULL) 
	{
		perror("Client::Client, gethostbyname");
		return;
	}

	if ((m_iSock = socket(AF_INET, SOCK_STREAM, 0)) == -1) 
	{
		perror("Client::Client, socket");
		return;
	}

	m_addrRemote.sin_family		= AF_INET;        
	m_addrRemote.sin_port		= htons(m_iPort);      
	m_addrRemote.sin_addr		= *((struct in_addr *) he->h_addr); 
	memset(&(m_addrRemote.sin_zero), 0, 8);

	if (connect(m_iSock, (struct sockaddr *) &m_addrRemote, sizeof(struct sockaddr)) == -1) 
	{
		perror("Client::Client, connect m_iSock");
		return;
	}

	if (pResult)
		*pResult = true;
}

Client::~Client()
{
#ifdef _WIN32
	WSACleanup();
#endif

	if (m_pBuffer)
	{
		free(m_pBuffer);
		m_pBuffer = NULL;
	}
	if (m_pBuffer2)
	{
		free(m_pBuffer2);
		m_pBuffer2 = NULL;
	}

}

// Send some integers over the wire
bool Client::SendInts(int* values, int size)
{
	int	i = 0;

	if (send(m_iSock, (char *) values, sizeof(int) * size, 0) == -1)
	{
		perror("Client::SendInts, send ints");              
		return false;
	}

	if (VERBOSE)
	{ 
		printf("Client: sending %d ints - ", size);                       
		for (i = 0; i < size; i++)
			printf("%d ", values[i]);
		printf("\n");
	}

#ifdef DEBUG_ACK
	if (!RecvAck())
		return false;
	if (!SendAck())
		return false;	
#endif

	return true;
}

bool Client::Close()
{
	if (shutdown(m_iSock, SD_BOTH) == -1)
		return false;
#ifndef _WIN32
	close(m_iSock);
#endif
	return true;
}

bool Client::RecvAck()
{
	char temp[1];
	int iTotal = 0;
	int iResult = 0;

	if (VERBOSE)
		printf("Waiting for ack...\n");

	while ((iTotal < 1) && (iResult != -1))
	{
		iResult = recv(m_iSock, temp, 1, 0);	
		iTotal += iResult;	
	}
	if (iResult == -1)
	{
		perror("Client::RecvAck, recv");
		return false;
	}

	if (VERBOSE)
		printf("Ack recieved.\n");

	return true;
}

bool Client::SendAck()
{
	char temp[1];
	temp[0] = 42;

	if (VERBOSE)
		printf("Sending ack...\n");

	if (send(m_iSock, temp, 1, 0) == -1)
	{
		perror("Client::SendAck, send");
		return false;
	}

	return true;
}