/*
 * User.h
 *
 *  Created on: Apr 22, 2011
 *      Author: johnsverre
 */

#ifndef USER_H_
#define USER_H_

#include <XnCppWrapper.h>
#include <vector>
#include <queue>
#include "UserPoints.h"
#include "FeatureSet.h"
#include <string>

using namespace std;

class User {
public:
	User();
	User(XnUserID user);
	virtual ~User();
	XnUserID getUserID() const;
	double calcSpeed(UserPoints up1, UserPoints up2) const;
	double calcAcceleration(UserPoints up1, UserPoints up2, UserPoints up3) const;
	double calcAngularVelocity(UserPoints up1, UserPoints up2, UserPoints up3) const;
	double calcBodyAlignment(UserPoints up) const;
	double calcDistance(UserPoints up) const;
	double calcHeading(UserPoints up1, UserPoints up2) const;
	void calcFeatures();
	void addUserPoints();
	int countUserPoints() const;
	int countFeatureSets() const;
	bool validUserPoints(UserPoints up) const;
	UserPoints getLatestUserPoints() const;
	FeatureSet getLatestFeatureSet() const;
	//void setOpeningDoor(bool isOpeningDoor);
	int getPositiveIndications() const;
	void setPositiveIndications(int nPositive);
	bool isDisappearing() const;
	bool getIntention() const;
	void setIntention(bool intention); 
	string toString() const; 
private:
	XnUserID xnid;
	vector<UserPoints> userPoints;
	vector<UserPoints> userPointTempBuffer;
	queue<FeatureSet> featureStack;
	double calcVectorAngle(double *v1, double *v2) const;
	bool angleDeg;
	int maxUserPoints;
	int maxFeatureSets;
	int positiveIndications;
	bool intention;
	//bool openingDoor;
};

#endif /* USER_H_ */
