/*
 * UserHandler.cpp
 *
 *  Created on: Apr 22, 2011
 *      Author: johnsverre
 */
#include "Client.h"
#include "UserHandler.h"
#include "User.h"
#include <iostream>
#include <XnCppWrapper.h>
#include <boost/thread.hpp>
#include "DecisionMaker.h"

using namespace xn;

UserHandler::UserHandler() {
	this->decisionMaker = new DecisionMaker();
}

UserHandler::~UserHandler() {
	delete this->decisionMaker;
}


void UserHandler::addUser(User u){

	if(u.getUserID() == 0)
		return;
	cout << "adding user.. \n" << u.toString() << endl;
	
	users.push_back(u);
	int size = users.size();
	for ( int i = 0 ; i < size ; i++ ) {
		cout << "user: " << users[i].countFeatureSets();	
	}
}


User UserHandler::getUser(int index){
	if(!users.empty() && users.size() > index){
		return users.at(index);
	}
	return User(0);
}

void UserHandler::removeUser(XnUserID userid){
	if(users.empty()) return;

	int size = users.size();

	for (int i = 0;i<size;i++){
		if(users[i].getUserID() == userid){
			cout << "removing user.. \n" << users[i].toString() << endl;
			users.erase(users.begin()+i);
			break;
		}
	}
	if(users.empty()) {
		bool bResult = false;
		int C[1];
		
		fflush(NULL);
		C[0] = 0;
	
		fflush(NULL);
  		Client mylink(5010, "Oyvind-PC", &bResult);

		if (!bResult)
		{
			printf("Failed to create Client object!\n");
		}
		mylink.SendInts(C, 1);
		fflush(NULL);
		mylink.Close();
		fflush(NULL);
	}
}

void UserHandler::runUserCalculations(){
	if(users.empty()) return;

	int size = users.size();

	for (int i = 0;i<size;i++){
		users[i].calcFeatures();
	}
}

void UserHandler::updateUserPoints(){
	extern UserGenerator g_UserGenerator;

	if(users.empty()) return;

	int size = users.size();


	for (int i = 0;i<size;i++){
		XnUserID uid = users[i].getUserID();
		users[i].addUserPoints();
	}
}

int UserHandler::getUserCount() const{
	return users.size();
}

void UserHandler::inferUserIntentions(XnUserID &openingDoor){
	if(users.empty()) return;

	int size = users.size();
	int oldAmountOfPositiveIntention = getAllUsersIntentions();

	for (int i = 0;i<size;i++){
		bool positiveIntention = decisionMaker->evaluateUser(users[i]);
		int nPositive = users[i].getPositiveIndications();

		if(positiveIntention){
			if(nPositive > 3){ // let user have multiple positive indications before opening
				users[i].setIntention(true);
			}else{
				int n = users[i].getPositiveIndications();
				nPositive++;
				users[i].setPositiveIndications(nPositive);
			}
		}else{
			nPositive--;
			users[i].setPositiveIndications(nPositive);
		}
	}

	bool bResult = false;
	int C[1];
		
	fflush(NULL);
	C[0] = getAllUsersIntentions();
	//Send data only if the number of positive intentions has changed
	//if(C[0] != oldAmountOfPositiveIntention) {
		fflush(NULL);
  		Client mylink(5010, "Oyvind-PC", &bResult);

		if (!bResult)
		{
			printf("Failed to create Client object!\n");
		}
		mylink.SendInts(C, 1);
		fflush(NULL);
		mylink.Close();
		fflush(NULL);
	//}  

}
int UserHandler::getAllUsersIntentions() const{
	if(users.empty()) return 0;

	int size = users.size();
	int intentions = 0;
	for(int i = 0;i<size ; i++){
		if(users[i].getIntention()){
			intentions++;		
		}	
	}
	return intentions;
}

	

