package sliding;

import java.io.*;
import java.net.*;
import java.util.ArrayList;

import org.npra.ven.*;
import org.osgi.framework.ServiceReference;

import sliding.Activator;
/**
 * Class representing the a server socket.
 * @author yvind
 *
 */
public class Server implements Runnable {

	static int BUFFSIZE = 128000; // how many bytes the incoming buffer can hold
	static int INT_SIZE = 1;

	byte data[];

	int port;
	ServerSocket server;
	Socket sock;
	Activator activator;

	protected boolean isStopped = false;

	BufferedInputStream input;
	BufferedOutputStream output;
   
	public Server(int p, int datap, Activator activator) throws IOException
	{
		port = p;
		this.activator = activator;
		try {
			server = new ServerSocket(port, 100);
		} catch ( IOException e ) {
			e.printStackTrace();
		}
		data = new byte[BUFFSIZE];
	}

    public void run()
    {
		byte rev[] = new byte[1];
		rev[0] = 0;
		
		ServiceReference reference = Activator.bc.getServiceReference(VenSend.class.getName());
		VenSend service = (VenSend)Activator.bc.getService(reference);	
		 while(!isStopped())
		 {
	            Socket clientSocket = null;
	            try {
	            	sock = server.accept();
	        		input = new BufferedInputStream(sock.getInputStream(), BUFFSIZE);
	        		output = new BufferedOutputStream(sock.getOutputStream(),BUFFSIZE);
	                clientSocket = sock;
	                
	                int C[];
	            	C = new int [1];
	                double numberOfPostiveIntentions =  RecvInts(C, 1);
	                if(numberOfPostiveIntentions != -1.0) 
	                {
	                	ArrayList<Double> intention=new ArrayList<Double>();
		    			intention.add((Double)numberOfPostiveIntentions);
		    			service.sendMessage("KINECT","INTENTION",intention);
	                }	
	            } catch (Exception e) {
	                if(isStopped()) 
	                {
	                    System.out.println("Server Stopped.") ;
	                    return;
	                }
	                throw new RuntimeException(
	                    "Error accepting client connection", e);
	            }
	            new Thread(
	                new WorkerRunnable(
	                    clientSocket, "Multithreaded Server")
	            ).start();
		 }
	     System.out.println("Server Stopped.");
   }
   /**
    * Recieves and converts the values from binary to integer.
    * @param val A list of values from the client socket
    * @param maxlen Maximum length of the integers
    * @return The number of positive pedestrian intentions
    * @throws IOException
    */
   public int RecvInts(int val[], int maxlen) throws IOException
   {
        int i;
        int numbytes = 0;
        int numberOfPositiveIntentions = -1;
		if (maxlen*INT_SIZE>BUFFSIZE)
		{
			System.out.println("Sending more ints then will fit in buffer!");
		}
		
		numbytes = input.read(data);
		String binary = "";
		for (i = 0; i < numbytes; i++) 
		{
			binary += data[i];
		}
		binary = binary.trim();
		if(!binary.equals(null)) 
		{
			String[] stringArray = binary.split("");

			StringBuffer result = new StringBuffer();
			for(i = 0; i < stringArray.length; i++) 
			{
				if(stringArray[i].equals("0")) 
				{
					break;
				} 
				else 
				{
					result.append(stringArray[i]); 
				}
			}
			String intentionString = result.toString();
			if(intentionString.equals("")) 
			{
				numberOfPositiveIntentions = 0;
			} 
			else 
			{
				if(stringArray != null && stringArray.length == 5)
				{
					try {
				 		numberOfPositiveIntentions = Integer.parseInt(intentionString);
					} catch (Exception e) {
						System.out.println("Server error" + intentionString);
					}
				}
			}
		} else numberOfPositiveIntentions = 0;

		SendAck();		
		return numberOfPositiveIntentions;
   }
 
   public void Close() throws IOException
   {
		sock.close();
   }
   private synchronized boolean isStopped() 
   {
       return this.isStopped;
   }

   private void SendAck() throws IOException
   {
		int ack;
		ack = 0;
		output.write(ack);
		output.flush();
   }

}

